/// ----------------------------------------
/// -------- SuperCollider Tutorial --------
/// -------------- Part I ------------------
/// ---------- basic concepts --------------

What is SuperCollider?

SuperCollider is both a language and an audio server.
Practically this means that to use SuperCollider, you
have to be aware, that it is built up out of two parts:
sclang - the language
scsynth - the server

Within the language, there are all kinds of ways available
to communicate with the server, but it is useful to know
that in the end, the two communicate with each other through
OSC - Open Sound Control, a network protocol, designed for 
sound (this means that the language and server could also be
running on different computers).

You are reading this file now in a text editor that interfaces
sclang, that is, from the text editor you can execute code in
the SuperCollider language.
Depending on the editor you are using (on MacOSX, Linux or Windows),
there are different Shortcuts to execute the code.
See the file [Shortcuts] for a list. Now to open this link, is a
first exercise to use the Help System of SuperCollider.
* Select the word Shortcuts between the square brackets
* On Mac: type <cmd-?>
* On Windows: type <F1>
* On Linux/Emacs: type  <C-c> <C-h>


The Help files can also be browsed in any webbrowser, as they are
formatted in HTML (as of ca. March 2007).
Within the Help files also several other tutorials can be found.

Help.gui

/// ---------- making some sound ------------

In order to make sound, first ensure that you have your audio
settings configured properly (see the file SoundSetup.scd)

Then we can start the server, by executing the following line
(see the shortcuts file again on how to do this).

s = Server.local.boot;

What happens on this line is the following:
Server.local returns the local server, i.e. an instance of scsynth
that will run on the local machine. The message .boot starts scsynth.
And finally we assign it to the variable named "s".
We could have assigned it to any other variable, but "s" is usually
used for the server as a convention. We will go in more detail on the
use of variables further on.
First we will go step by step through the process of creating a sine based
synthesizer.

This will create a sine tone at 440 Hz:
{ SinOsc.ar( 440, 0, 0.025 ) }.play; // execute this line

To stop the sound, type (see shortcuts)
cmd-. or   alt-. or   C-c C-s

Now what this line does, is the following:
It uses the Unit Generator (or UGen) called SinOsc at the AudioRate (.ar),
at the frequency 440 (the first argument), with phase offset 0 and at
amplitude 0.3. We can look at the Help file of SinOsc if we want more
information on its arguments.
Around the definition of our SinOsc, there are curly brackets followed by
the message .play.
The curly brackets indicate that we have defined a Function. The message
.play means that we want to play the function. As we have defined something
built up from UGens inside the function, sclang understands that we probably
want to create a SynthDef and send it to the server to play it.
(more information on the .play method can be found in the Help file of [Function]).
Using Function.play is a useful method if we quickly want to try out something,
however, when we want to design a Synth, we need to write a SynthDef:

( // this defines the same synth as created by the function, and sends its definition
// to the server.
SynthDef( \simplesine, {
	Out.ar( 0, SinOsc.ar( 440, 0, 0.025 ) );
}).load(s);
) // again select the block between the brackets and execute

// to start it we execute:
x = Synth.new( \simplesine );
// this creates an instance of the Synth on the server

// to stop:
x.free;

Now, a sine tone at 440 Hz is quite boring. So we adapt the SynthDef, so we
can vary its arguments:

( 
SynthDef( \simplesine, { arg out=0, freq=440, amp=0.025;
	Out.ar( out, SinOsc.ar( freq, 0, amp ) );
}).load(s);
)
We have now given the SynthDef \simplesine three arguments: the output channel (out),
the frequency (freq) and the amplitude (amp). We have also given each of these
arguments default values.

// we can now start the synth again and then change the values:
x = Synth.new( \simplesine );
x.set(\freq, 260); // change the frequency
x.set(\amp, 0.01 ); // change the amplitude
x.set(\out, 0 ); // change the output channel
x.free; // stop

// we can also start the synth with a set of values:
x = Synth.new( \simplesine, [\freq,600,\amp,0.01,\out,1] );
x = Synth.new( \simplesine, [\freq,600,\out,1] ); // this sets two arguments, but leaves amp at the default.
x.free;


Now, a continuous sine wave is quite boring, so we want to create
an envelope for it:

( 
SynthDef( \simplesine, { arg out=0, freq=440, amp=0.3;
	Out.ar( out, SinOsc.ar( freq, 0, amp ) * EnvGen.kr( Env.perc( 0.05, 0.95, 1, -3 ), doneAction: 2 ) );
}).load(s);
)

Env.perc( 0.05, 0.95, 1, -3 ).plot

This creates a percussive envelope, with an attack time of 0.05 seconds, and a decay of 0.95. The arguments "1" and "-3" to Env.perc define the peakLevel and curvature. EnvGen.kr is the actual UGen that applies the Envelope to the sound (note here the .kr, meaning c(k)ontrol rate; below is an explanation of these rates).
After the Envelope has been played, it frees the synth when it is done.

x = Synth.new( \simplesine, [\freq,600,\amp,0.1,\out,1] );

Next we want to be able to change the duration:

( 
SynthDef( \simplesine, { arg out=0, freq=440, amp=0.3, dur=1;
	Out.ar( out, SinOsc.ar( freq, 0, amp ) * EnvGen.kr( Env.perc( 0.05, 0.95, 1, -3 ), timeScale: dur, doneAction: 2 ) );
}).store;
)

x = Synth.new( \simplesine, [\freq,600,\amp,0.1,\out,1,\dur,0.5] );
x = Synth.new( \simplesine, [\freq,600,\amp,0.1,\out,1,\dur,5] );


(
Task({
	100.do({ 
   	Synth.new( \simplesine, [\freq,600.rand + 500,\amp,0.1,\out,1,\dur,0.5] );
		0.25.wait;
	});
}).play;
)

Shortcuts

p = Pbind( \instrument, \simplesine, \dur, Pseq( [0.1,0.4,0.8,0.2],5), \freq, Pwhite(1,8) * 100 ).trace.play;
p.stop

/// ----- variables ----------
Within the language, it is often useful to assign values, objects or functions
to variables. In the default environment, we have the 26 letters of the alphabet
at our disposal as variables which are always available. That is, we can assign
something to them in one line of code, execute that line, write a next line of 
code using that variable, and the interpreter will still know about that variable.

For example:

a = 5; // execute this line
b = a + 2; // then execute this line

As opposed to:
var alpha; alpha = 5; // execute this line
var beta; beta = alpha + 3; // then, execute this line

The last example would have worked if we had written:
(
var alpha,beta;
alpha = 5;
beta = alpha + 3;
) // select the whole block between the brackets and execute

In this example, we executed a whole block of code, in which we first
declared the two variables named "alpha" and "beta", using the word "var",
then assigned a value to "alpha", and then calculated the value of "beta"
by adding 3 to "alpha".

There are two tricks that can be used to get around this:

the first is to use a variable name that starts with ~:

~alpha = 5; // execute this line
~beta = ~alpha + 3; // then, execute this line

~beta

This declares variables as Environment variables in the defaul Environment.
This works, but may give problems with certain names, which are already
defined in the environment. Dont worry about this now, but keep it in mind.

Another trick is to use a dictionary of an Event like this:

q = (); 
q.alpha = 5;
q.beta = q.alpha + 3;
q.gamma = { q.alpha + 3 };
q.alpha = 6;
q.gamma.value;

This way, you can create "pseudo" classes.

/// -------- arguments ------------
Arguments are used inside functions and are defined with the word "arg".
An alternate (shorter) way of writing this is:

arg a=4,b=2,c=5;

|a=4,b=2,c=5|

Both are valid code, and both are used throughout the examples.

/// -------- UGen rates -----------
UGens can have two different rates: audio rate (.ar) and control rate (.kr).
Audio rate runs at whatever sampling rate the server is set to (normally 44.1 kHz or 48 kHz)
Control rate runs at once each block, and thus depends on the block size the server is set to (minimum is 64 samples).
Both of these are set in the ServerOptions (see Help file for further info).

Generally, if a UGen does not need to be updated all that often, use the control rate, as it will save CPU time.

/// (May 2007, Marije Baalman)